<?php
namespace Teknisa\MAN\Services; 

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class Operador {

    protected $entityManager;
    protected $criptografia;
    protected $operador;
    protected $novoCodigo;
    protected $nrparcnegocio;
    protected $nrorganizacaooper;
    protected $now;

    public function __construct(EntityManager $entityManager, Criptografia $criptografia, NovoCodigo $novoCodigo) {
        $this->entityManager = $entityManager;
        $this->criptografia = $criptografia;
        $this->novoCodigo = $novoCodigo;
        $this->now = new \DateTime();
    }

    private function atualizaSenha($senha) {
        $senhaWeb = $this->criptografia->retornaSenhaWeb($senha);
        $senhaDelphi = $this->criptografia->retornaSenhaDelphi($senha);
        $this->operador->setCdsenhaoperweb($senhaWeb);
        $this->operador->setCdsenhoper($senhaDelphi);
        $this->entityManager->persist($this->operador);
        $this->entityManager->flush();
    }

    public function inserirParceiroDeNegocio($operador) {
        $nrinscricaoparc = "00000000".$operador['cdoperador'];
        $parcnegocio = new \Geral\Entities\Parcnegocio();
        $this->nrparcnegocio = $this->novoCodigo->getCodeByTableName('PARCNEGOCIO', $operador['nrorgtrab']);
        $parcnegocio->setNrparcnegocio($this->nrparcnegocio);
        $parcnegocio->setNrorg($operador['nrorgtrab']);
        $parcnegocio->setCdtipoparcprincipal('OPERADOR');
        $parcnegocio->setNmprincipalparc($operador['nmoperador']);
        $parcnegocio->setNmsecundariparc($operador['nmoperador']);
        $parcnegocio->setDtnascifundparc(null);
        $parcnegocio->setCdtipoinscricao('LIVRE');
        $parcnegocio->setNrinscricaoparc($nrinscricaoparc);
        $parcnegocio->setIdativo('S');
        $parcnegocio->setDtinclusao($this->now);
        $parcnegocio->setDtultatu($this->now);
        $parcnegocio->setIdparcfundido('N');
        $parcnegocio->setIdpessoafisica('S');
        $parcnegocio->setNrorginclusao($operador['nrorginclusao']);
        $parcnegocio->setCdoperinclusao($operador['cdoperinclusao']);
        $this->entityManager->persist($parcnegocio);
        $this->entityManager->flush();
    }

    public function inserirOperador($operador) {
        $senhaWeb = $this->criptografia->retornaSenhaWeb($operador['senha']);
        $senhaDelphi = $this->criptografia->retornaSenhaDelphi($operador['senha']);
        $obj = new \Geral\Entities\Operador();
        $obj->setCdoperador($operador['cdoperador']);
        $obj->setNrparcnegocio($this->nrparcnegocio);
        $obj->setNmoperador($operador['nmoperador']);
        $obj->setNrorgtrab($operador['nrorgtrab']);
        $obj->setDsemailoper($operador['dsemailoper']);
        $obj->setCdsenhoper($senhaDelphi);
        $obj->setCdsenhaoperweb($senhaWeb);
        $obj->setNrorg($operador['nrorgtrab']);
        $obj->setNrestruttrab($operador['nrestruturam']);
        $obj->setDtinclusao($this->now);
        $obj->setDtultatu($this->now);
        $obj->setCdoperinclusao($operador['cdoperinclusao']);
        $this->entityManager->persist($obj);
        $this->entityManager->flush();
    }

    public function inserirPerfilAcesso($operador) {
        $nrperfacessparc = $this->novoCodigo->getCodeByTableName('PERFACESSPARC', $operador['nrorgtrab']);
        $perfacessparc = new \Geral\Entities\Perfacessparc();
        $perfacessparc->setNrperfacessparc((int)$nrperfacessparc);
        $perfacessparc->setNrparcnegocio($this->nrparcnegocio);
        $perfacessparc->setNrperfilacesso($operador['nrperfilacesso']);
        $perfacessparc->setNrorg($operador['nrorgtrab']);
        $perfacessparc->setIdativo('S');
        $perfacessparc->setDtinclusao($this->now);
        $perfacessparc->setDtultatu($this->now);
        $perfacessparc->setCdoperinclusao($operador['cdoperinclusao']);
        $this->entityManager->persist($perfacessparc);
        $this->entityManager->flush();
    }

    public function verificaEmail($dsemailoper, $nrorgtrab) {
        $this->operador = $this->entityManager->getRepository('\Geral\Entities\Operador')->findOneBy(
            array(
                'dsemailoper' => $dsemailoper,
                'nrorgtrab' => $nrorgtrab
            )
        );
        if($this->operador) {
            throw new \Exception("Email informado já existe.");
        }
    }

    public function retornarOperadores($nrestruttrab, $nrorgtrab, $idoperativo) {
        $array = array();
        $operadores = $this->entityManager->getRepository('\Geral\Entities\Operador')->findBy(
            array(
                'nrestruttrab' => $nrestruttrab,
                'nrorgtrab' => $nrorgtrab,
                'idoperativo' => $idoperativo
            )
        );
        foreach ($operadores as $operador) {
            $unidades = $this->entityManager->getRepository('\Geral\Entities\Filial')
                ->retornaFilialPorOperador($operador->getCdoperador());
            $nmunidade = null;
            $cdunidade = null;
            foreach ($unidades as $unidade) {
                if ($unidade['NRESTRUTURAM'] == $operador->getNrestruttrab()) {
                    $nmunidade = $unidade['NMFILIAL'];
                    $cdunidade = $unidade['CDFILIAL'];
                }
            }
            $array[] = array(
                'CDOPERADOR' => $operador->getCdoperador(),
                'NMUNIDADE' => $nmunidade,
                'CDUNIDADE' => $cdunidade,
                'CDORGANIZACAO' => $operador->getNrorgtrab(),
                'CDESTRUTURA' => $operador->getNrestruttrab(),
                'NMOPERADOR' => $operador->getNmoperador(),
                'UNIDADES' => $unidades
            );
        }
        return $array;
    }

    public function verificaCodigoOperador($cdoperador, $nrorgtrab) {
        $this->operador = $this->entityManager->getRepository('\Geral\Entities\Operador')->findOneBy(
            array(
                'cdoperador' => $cdoperador,
                'nrorgtrab' => $nrorgtrab
            )
        );
        if($this->operador) {
            throw new \Exception("Código informado já existe.");
        }
    }

    public function inserirOrganizacao($operador) {
        $this->operador = $this->entityManager->getRepository('\Geral\Entities\Organizacaooper')->findOneBy(
            array(
                'cdoperador' => $operador['cdoperador'],
                'nrorgrel' => $operador['nrorgtrab']
            )
        );
        if(!$this->operador) {
            $this->nrorganizacaooper = $this->novoCodigo->getCodeByTableName('ORGANIZACAOOPER', $operador['nrorgtrab']);
            $organizacaooper = new \Geral\Entities\Organizacaooper();
            $organizacaooper->setNrorganizacaooper($this->nrorganizacaooper);
            $organizacaooper->setNrorgrel($operador['nrorgtrab']);
            $organizacaooper->setCdoperador($operador['cdoperador']);
            $organizacaooper->setDtinclusao($this->now);
            $organizacaooper->setDtultatu($this->now);
            $this->entityManager->persist($organizacaooper);
            $this->entityManager->flush();
        }
    }

    public function inserirEstruturas($operador) {
        foreach ($operador['unidadesRelacionadas'] as $unidadeRelacionada) {
            $estrutura = $this->entityManager->getRepository('\Geral\Entities\Estruturaoper')->findOneBy(
                array(
                    'nrestruturamrel' => $unidadeRelacionada['NRESTRUTURAM'],
                    'nrorg' => $operador['nrorgtrab']
                )
            );
            if(!$estrutura) {
                $nrestruturaoper = $this->novoCodigo->getCodeByTableName('ESTRUTURAOPER', $operador['nrorgtrab']);
                $estruturaoper = new \Geral\Entities\Estruturaoper();
                $estruturaoper->setNrestruturaoper($nrestruturaoper);
                $estruturaoper->setNrorganizacaooper($this->nrorganizacaooper);
                $estruturaoper->setNrestruturamrel($unidadeRelacionada['NRESTRUTURAM']);
                $estruturaoper->setDtinclusao($this->now);
                $estruturaoper->setDtultatu($this->now);
                $estruturaoper->setCdoperinclusao($operador['cdoperinclusao']);
                $estruturaoper->setNrorg($operador['nrorgtrab']);
                $estruturaoper->setIdativo('S');
                $this->entityManager->persist($estruturaoper);
                $this->entityManager->flush();
            }
        }
    }

    public function inserirFiliaisOperador($operador) {
        foreach ($operador['unidadesRelacionadas'] as $unidadeRelacionada) {
            $filial = $this->entityManager->getRepository('\Geral\Entities\Filioper')->findOneBy(
                array(
                    'cdoperador' => $operador['cdoperador'],
                    'cdfilial' => $unidadeRelacionada['CDFILIAL'],
                    'nrorg' => $operador['nrorgtrab']
                )
            );
            if(!$filial) {
                $filioper = new \Geral\Entities\Filioper();
                $filioper->setCdoperador($operador['cdoperador']);
                $filioper->setCdfilial($unidadeRelacionada['CDFILIAL']);
                $filioper->setNrorg($operador['nrorgtrab']);
                $filioper->setDtinclusao($this->now);
                $filioper->setDtultatu($this->now);
                $filioper->setCdoperinclusao($operador['cdoperinclusao']);
                $filioper->setIdativo('S');
                $this->entityManager->persist($filioper);
                $this->entityManager->flush();
            }
        }
    }

    public function salvarAdicao($operador) {
        $this->verificaCodigoOperador($operador['cdoperador'], $operador['nrorgtrab']);
        $this->verificaEmail($operador['dsemailoper'], $operador['nrorgtrab']);
        $this->inserirParceiroDeNegocio($operador);
        $this->inserirOperador($operador);
        $this->inserirPerfilAcesso($operador);
        $this->inserirOrganizacao($operador);
        $this->inserirEstruturas($operador);
        $this->inserirFiliaisOperador($operador); //temporario
    }

    public function salvarEdicao($operador) {
        $this->operador = $this->entityManager->getRepository('\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $operador['cdoperador'],
                    'nrorgtrab' => $operador['nrorgtrab']
                )
            );
        if($this->operador) {
            $this->operador->setNmoperador($operador['nmoperador']);
            $this->operador->setNrestruttrab($operador['nrestruttrab']);
            $this->entityManager->persist($this->operador);
            $this->entityManager->flush();
            $this->inserirEstruturas($operador);
            $this->inserirFiliaisOperador($operador);
        }else{
            throw new \Exception("Operador não existe.");
        }
    }

    public function alterarSenhaOperador($cdoperador, $nrorgtrab, $senha, $senhaAtual) {
        $this->operador = $this->entityManager->getRepository('\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $cdoperador,
                    'nrorgtrab' => $nrorgtrab
                )
            );
        if($this->operador->getCdsenhaoperweb() === MD5($senhaAtual)) {
            $this->atualizaSenha($senha);
        }else{
            throw new \Exception("Senha inválida.");
        }
    }

    public function alterarIdiomaOperador($cdoperador, $nrorgtrab, $ididioma) {
        $this->operador = $this->entityManager->getRepository('\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $cdoperador,
                    'nrorgtrab' => $nrorgtrab
                )
            );
        $this->operador->setIdidioma($ididioma);
        $this->entityManager->persist($this->operador);
        $this->entityManager->flush();
    }

    public function alterarEstrutura($cdoperador, $nrestruturam, $nrorgtrab) {
        $operador = $this->entityManager->getRepository('\Geral\Entities\Operador')
            ->findOneBy(
                array(
                    'cdoperador' => $cdoperador,
                    'nrorgtrab' => $nrorgtrab
                )
            );
        $operador->setNrestruttrab($nrestruturam);
        $operador->setNrorgtrab($nrorgtrab);
        $this->entityManager->persist($operador);
        $this->entityManager->flush();
    }

    public function getOperadorByFilial($DSEMAILOPER, $CDFILIAL){
        $params = array(
            ':DSEMAILOPER' => $DSEMAILOPER,
            ':CDFILIAL' => $CDFILIAL
        );
        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_OPERADOR_BY_FILIAL, $params);
    }

    public function getOperadorEmail($DSEMAILOPER){
        $params = array(
            ':DSEMAILOPER' => $DSEMAILOPER      
        );

        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_OPERADOR_EMAIL, $params);
    }

    public function defineSenha($CDSENHAOPERMD5, $DSEMAILOPER){
        $params = array(
            ':CDSENHAOPERMD5' => $CDSENHAOPERMD5,
            ':DSEMAILOPER' => $DSEMAILOPER          
        );

        return $this->entityManager->getConnection()->executeQuery(Utilities\Query::UPDATE_CDSENHAOPERMD5_EMAIL, $params);
    }

    public function getSenhaOperEmail($DSEMAILOPER){
        $params = array(
            ':DSEMAILOPER' => $DSEMAILOPER          
        );
        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_SENHA_OPERADOR_EMAIL, $params);
    }

    public function getFilial ($CDOPERADOR) {
        $params = array(
            ':CDOPERADOR' => $CDOPERADOR
        );

        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_FILIAL_OPERADOR, $params);
    }

    public function getLojasByFilialAndOperador ($CDFILIAL, $CDOPERADOR) {
        $params = array(
            ':CDFILIAL'    => $CDFILIAL,
            ':CDOPERADOR'  => $CDOPERADOR
        );

        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_LOJAS_BY_FILIAL_OPERADOR, $params);
    }

    public function getCaixaByLojaAndFilial ($CDFILIAL, $CDLOJA) {
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':CDLOJA'   => $CDLOJA
        );

        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_CAIXA_BY_FILIAL_LOJA, $params);
    }
    public function getOperadores(){

        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_OPERADORES);
    }

    public function updateHashAlteraSenha ($CDOPERADOR, $DSHASHALTERASENHA) {
        $params = array(
            ':CDOPERADOR'        => $CDOPERADOR,
            ':DSHASHALTERASENHA' => $DSHASHALTERASENHA
        );
        $this->entityManager->getConnection()->executeQuery(Utilities\Query::UPDATE_HASH_ALTERASENHA, $params);
    }

    public function getHash ($CDOPERADOR) {
        try {
            $retorno = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_HASH_BY_OPERADOR, array(':CDOPERADOR' => $CDOPERADOR));
            return count($retorno) > 0 ? $retorno[0]['DSHASHALTERASENHA'] : null;
        } catch (\Exception $e) {
            return null;
        }
    }

    public function updatePassword($CDOPERADOR, $CDSENHAOPERMD5) {
        $params = array(
            ':CDOPERADOR' => $CDOPERADOR,
            ':CDSENHAOPERMD5' => $CDSENHAOPERMD5    
        );
        $this->entityManager->getConnection()->executeQuery(Utilities\Query::UPDATE_PASSWORD, $params);
    }

    public function getPerfilOper($NRPARCNEGOCIO, $NRORG){
        $params = array('NRPARCNEGOCIO' =>$NRPARCNEGOCIO,
                        'NRORG' =>$NRORG);
        $retorno = $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_PERFILOPER, $params);
    }

    public function getPerfilAssociado($CDOPERADOR){
        $lista = null;
        $MIN = 10000; // maximo de elementos da tabela de regras
        $RETORNO = null;
        $params = array(
            ':CDOPERADOR' => $CDOPERADOR,
        );
        $PERFIS = $this->entityManager->getConnection()->fetchAll(Utilities\Query::BUSCA_PERFILOPER, $params);
        foreach ($PERFIS as $key) {
            $PARAM['GROUP_ID'] = $key['NRPERFILACESSO'];
            $GROUPID =  $key['NRPERFILACESSO'];
            $RULES = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GETRULE_OPERADOR,$PARAM);
            $QTRULES = count($RULES); 
            /*
                O grupo com menor quantidade de regras é o mais abrangente e deve ser retornado.
             */
            if($QTRULES < $MIN){
                $MIN = $QTRULES;
                $RETORNO = $GROUPID;
            }
        }
        return $RETORNO;
    }

    public function getRegraElementOper($GROUPID){

        return $this->entityManager->getConnection()->fetchAll(Utilities\Query::GETRULE_OPERADOR,$GROUPID);
    }

}